"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GenerateAction = void 0;
const fs_1 = require("fs");
const path_1 = require("path");
const abstract_action_1 = require("../lib/common/abstract.action");
const executor_1 = require("../lib/executor");
const db_datasource_1 = require("../lib/utils/db.datasource");
const is_1 = require("../lib/utils/is");
const writer_1 = require("../lib/writer");
const fs_extra_1 = require("fs-extra");
class GenerateAction extends abstract_action_1.AbstractAction {
    handle(inputs, options) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const database = inputs.find((e) => e.name === 'database')
                    .value;
                const host = options.find((e) => e.name === 'host').value;
                const username = options.find((e) => e.name === 'username')
                    .value;
                const port = options.find((e) => e.name === 'port').value;
                const password = options.find((e) => e.name === 'password')
                    .value;
                const output = options.find((e) => e.name === 'output').value;
                // init mysql connection
                yield db_datasource_1.DBDataSource.getInstance().init({
                    database,
                    host,
                    username,
                    port,
                    password,
                });
                // test connection
                const [{ version }] = yield new executor_1.VersionExecutor().exec();
                console.log(`Mysql connection is successful. database version is ${version}`);
                console.log('\nLooking for all tables in the database:');
                const tables = yield new executor_1.TablesExecutor(database).exec();
                console.table(tables);
                // handle gen path
                let path = process.cwd();
                if ((0, is_1.isString)(output) && output) {
                    // handle relative path
                    path = output.startsWith('.') ? (0, path_1.resolve)(path, output) : output;
                    // ensure dir created
                    (0, fs_extra_1.ensureDirSync)(path);
                }
                // create stream
                const sb = (0, fs_1.createWriteStream)((0, path_1.join)(path, `${database}.md`), {
                    start: 0,
                    encoding: 'utf8',
                });
                const headerWriter = new writer_1.HeaderWriter(sb);
                // docs header
                yield headerWriter.write(`${database}数据库文档`, 1, false, '<a name="返回顶部"></a>');
                // docs outline
                yield headerWriter.write('大纲', 2);
                yield new writer_1.OutlineWriter(sb).write(tables);
                // tables
                for (let i = 0; i < tables.length; i++) {
                    yield headerWriter.write(`${tables[i].name}[↑](#返回顶部)<a name="${tables[i].name}"></a>`, 2, true, tables[i].comment ? `> 表注释: ${tables[i].comment}` : undefined);
                    yield new writer_1.DescWriter(sb).write(database, tables[i]);
                }
                // writing end
                sb.end();
                console.log('√ Document generated!');
            }
            finally {
                // destroy connection
                db_datasource_1.DBDataSource.getInstance().destroy();
            }
        });
    }
}
exports.GenerateAction = GenerateAction;
