"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DescWriter = void 0;
const abstract_writer_1 = require("../common/abstract.writer");
const executor_1 = require("../executor");
const colTitles = ['字段', '类型', '空', '默认值', 'EXTRA', '注释'];
const colKeys = [
    'name',
    'type',
    'isNullable',
    'defaultValue',
    'extra',
    'comment',
];
const indexdTitles = [
    '键名',
    '类型',
    '唯一',
    '字段',
    '基数',
    '排序规则',
    '空',
    '注释',
];
const indexdKeys = [
    'indexName',
    'type',
    'nonunique',
    'colName',
    'cardinality',
    'collation',
    'nullable',
    'comment',
];
class DescWriter extends abstract_writer_1.AbstractWriter {
    write(db, table) {
        return __awaiter(this, void 0, void 0, function* () {
            const text = [];
            // columns
            const columns = yield new executor_1.ColumnsExecutor(db, table.name).exec();
            text.push(['', ...colTitles, ''].join('|'));
            text.push(['', ...colTitles.map(() => ':---:'), ''].join('|'));
            columns.forEach((c) => {
                const values = colKeys.map((k) => c[k]);
                text.push(['', ...values, ''].join('|'));
            });
            // indexd
            const indexd = yield new executor_1.IndexdExecutor(db, table.name).exec();
            if (indexd.length > 0) {
                // new line
                text.push(...['', '**索引**', '']);
                text.push(['', ...indexdTitles, ''].join('|'));
                text.push(['', ...indexdTitles.map(() => ':---:'), ''].join('|'));
                indexd.forEach((c) => {
                    const values = indexdKeys.map((k) => {
                        if (k === 'nonunique') {
                            return c[k] === 0 ? 'YES' : 'NO';
                        }
                        else if (k === 'nullable') {
                            return c[k] === 'YES' ? 'YES' : 'NO';
                        }
                        return c[k];
                    });
                    text.push(['', ...values, ''].join('|'));
                });
            }
            // new line
            text.push('');
            this.stream.write(text.join('\n'));
        });
    }
}
exports.DescWriter = DescWriter;
